@@ -33,6 +33,7 @@ from nuitka.constants.Serialization import ConstantAccessor
 from nuitka.freezer.IncludedDataFiles import (
     addIncludedDataFilesFromFileOptions,
     addIncludedDataFilesFromPackageOptions,
+    addIncludedDataFilesFromPlugins,
     copyDataFiles,
 )
 from nuitka.freezer.IncludedEntryPoints import (
@@ -59,20 +60,19 @@ from nuitka.PythonFlavors import (
     isApplePython,
     isDebianPackagePython,
     isFedoraPackagePython,
-    isMSYS2MingwPython,
     isNuitkaPython,
     isPyenvPython,
 )
 from nuitka.PythonVersions import (
     getPythonABI,
     getSupportedPythonVersions,
-    getSystemPrefixPath,
     python_version,
     python_version_str,
 )
 from nuitka.Tracing import general, inclusion_logger
 from nuitka.tree import SyntaxErrors
-from nuitka.utils import InstanceCounters, MemoryUsage
+from nuitka.tree.ReformulationMultidist import createMultidistMainSourceCode
+from nuitka.utils import InstanceCounters
 from nuitka.utils.Execution import (
     callProcess,
     withEnvironmentVarOverridden,
@@ -81,17 +81,17 @@ from nuitka.utils.Execution import (
 from nuitka.utils.FileOperations import (
     changeFilenameExtension,
     deleteFile,
-    getDirectoryRealPath,
     getExternalUsePath,
     openTextFile,
     removeDirectory,
     resetDirectory,
 )
 from nuitka.utils.Importing import getSharedLibrarySuffix
+from nuitka.utils.MemoryUsage import reportMemoryUsage, showMemoryTrace
 from nuitka.utils.ModuleNames import ModuleName
 from nuitka.utils.ReExecute import callExecProcess, reExecuteNuitka
 from nuitka.utils.StaticLibraries import getSystemStaticLibPythonPath
-from nuitka.utils.Utils import getArchitecture, isMacOS, isWin32Windows
+from nuitka.utils.Utils import isMacOS, isWin32Windows
 from nuitka.Version import getCommercialVersion, getNuitkaVersion
 
 from . import ModuleRegistry, Options, OutputDirectories
@@ -104,7 +104,7 @@ from .build.SconsInterface import (
 )
 from .code_generation import CodeGeneration, LoaderCodes, Reports
 from .finalizations import Finalization
-from .freezer.Onefile import packDistFolderToOnefile
+from .freezer.Onefile import getCompressorPython, packDistFolderToOnefile
 from .freezer.Standalone import (
     checkFreezingModuleSet,
     copyDllsUsed,
@@ -112,28 +112,54 @@ from .freezer.Standalone import (
 )
 from .optimizations.Optimization import optimizeModules
 from .pgo.PGO import readPGOInputFile
-from .Reports import writeCompilationReport
+from .reports.Reports import writeCompilationReports
 from .tree.Building import buildMainModuleTree
 from .tree.SourceHandling import writeSourceCode
 from .TreeXML import dumpTreeXMLToFile
 
 
-def _createNodeTree(filename):
+def _setupFromMainFilenames():
+    main_filenames = Options.getMainEntryPointFilenames()
+    for filename in main_filenames:
+        # Inform the importing layer about the main script directory, so it can use
+        # it when attempting to follow imports.
+        Importing.addMainScriptDirectory(
+            main_dir=os.path.dirname(os.path.abspath(filename))
+        )
+
+
+def _createMainModule():
     """Create a node tree.
 
-    Turn that source code into a node tree structure. If recursion into
-    imported modules is available, more trees will be available during
-    optimization, or immediately through recursed directory paths.
+    Turn that source code into a node tree structure. If following into
+    imported modules is allowed, more trees will be available during
+    optimization, or even immediately through forcefully included
+    directory paths.
 
     """
-
     # Many cases to deal with, pylint: disable=too-many-branches
 
+    Plugins.onBeforeCodeParsing()
+
+    main_filenames = Options.getMainEntryPointFilenames()
+
     # First, build the raw node tree from the source code.
-    main_module = buildMainModuleTree(
-        filename=filename,
-        is_main=not Options.shallMakeModule(),
-    )
+    if len(main_filenames) > 1:
+        main_module = buildMainModuleTree(
+            # TODO: Should not be given.
+            filename=main_filenames[0],
+            is_main=True,
+            source_code=createMultidistMainSourceCode(main_filenames),
+        )
+
+    else:
+        main_module = buildMainModuleTree(
+            filename=main_filenames[0],
+            is_main=not Options.shallMakeModule(),
+            source_code=None,
+        )
+
+    OutputDirectories.setMainModule(main_module)
 
     # First remove old object files and old generated files, old binary or
     # module, and standalone mode program directory if any, they can only do
@@ -163,6 +189,9 @@ def _createNodeTree(filename):
             path=OutputDirectories.getResultFullpath(onefile=True), must_exist=False
         )
 
+        # Also make sure we inform the user in case the compression is not possible.
+        getCompressorPython()
+
     # Second, do it for the directories given.
     for plugin_filename in Options.getShallFollowExtra():
         Recursion.checkPluginPath(plugin_filename=plugin_filename, module_package=None)
@@ -171,13 +200,13 @@ def _createNodeTree(filename):
         Recursion.checkPluginFilenamePattern(pattern=pattern)
 
     for package_name in Options.getMustIncludePackages():
-        package_name, package_directory, kind = Importing.locateModule(
+        package_name, package_directory, _module_kind, finding = Importing.locateModule(
             module_name=ModuleName(package_name),
             parent_package=None,
             level=0,
         )
 
-        if kind != "absolute":
+        if finding != "absolute":
             inclusion_logger.sysexit(
                 "Error, failed to locate package '%s' you asked to include."
                 % package_name
@@ -189,13 +218,13 @@ def _createNodeTree(filename):
         )
 
     for module_name in Options.getMustIncludeModules():
-        module_name, module_filename, kind = Importing.locateModule(
+        module_name, module_filename, _module_kind, finding = Importing.locateModule(
             module_name=ModuleName(module_name),
             parent_package=None,
             level=0,
         )
 
-        if kind != "absolute":
+        if finding != "absolute":
             inclusion_logger.sysexit(
                 "Error, failed to locate module '%s' you asked to include."
                 % module_name.asString()
@@ -209,6 +238,9 @@ def _createNodeTree(filename):
     Plugins.onModuleInitialSet()
 
     # Then optimize the tree and potentially recursed modules.
+    # TODO: The passed filename is really something that should come from
+    # a command line option, it's a filename for the graph, which might not
+    # need a default at all.
     optimizeModules(main_module.getOutputFilename())
 
     # Freezer may have concerns for some modules.
@@ -540,7 +572,7 @@ def _runPythonPgoBinary():
     return pgo_filename
 
 
-def runSconsBackend(quiet):
+def runSconsBackend():
     # Scons gets transported many details, that we express as variables, and
     # have checks for them, leading to many branches and statements,
     # pylint: disable=too-many-branches,too-many-statements
@@ -556,15 +588,8 @@ def runSconsBackend(quiet):
         "experimental": ",".join(Options.getExperimentalIndications()),
         "trace_mode": asBoolStr(Options.shallTraceExecution()),
         "python_version": python_version_str,
-        "target_arch": getArchitecture(),
-        "python_prefix": getDirectoryRealPath(getSystemPrefixPath()),
         "nuitka_src": getSconsDataPath(),
-        "module_count": "%d"
-        % (
-            1
-            + len(ModuleRegistry.getDoneModules())
-            + len(ModuleRegistry.getUncompiledNonTechnicalModules())
-        ),
+        "module_count": "%d" % len(ModuleRegistry.getDoneModules()),
     }
 
     if Options.isLowMemory():
@@ -589,9 +614,6 @@ def runSconsBackend(quiet):
     if isFedoraPackagePython():
         options["fedora_python"] = asBoolStr(True)
 
-    if isMSYS2MingwPython():
-        options["msys2_mingw_python"] = asBoolStr(True)
-
     if isApplePython():
         options["apple_python"] = asBoolStr(True)
 
@@ -676,7 +698,6 @@ def runSconsBackend(quiet):
 
         result = runScons(
             options=options,
-            quiet=quiet,
             env_values=env_values,
             scons_filename="Backend.scons",
         )
@@ -700,7 +721,6 @@ def runSconsBackend(quiet):
 
             result = runScons(
                 options=options,
-                quiet=quiet,
                 env_values=env_values,
                 scons_filename="Backend.scons",
             )
@@ -716,7 +736,6 @@ def runSconsBackend(quiet):
     result = (
         runScons(
             options=options,
-            quiet=quiet,
             env_values=env_values,
             scons_filename="Backend.scons",
         ),
@@ -801,12 +820,12 @@ def compileTree():
     if not Options.shallOnlyExecCCompilerCall():
         general.info("Generating source code for C backend compiler.")
 
-        if Options.isShowProgress() or Options.isShowMemory():
-            general.info(
-                "Total memory usage before generating C code: {memory}:".format(
-                    memory=MemoryUsage.getHumanReadableProcessMemoryUsage()
-                )
-            )
+        reportMemoryUsage(
+            "before_c_code_generation",
+            "Total memory usage before generating C code:"
+            if Options.isShowProgress() or Options.isShowMemory()
+            else None,
+        )
 
         # Now build the target language code for the whole tree.
         makeSourceDirectory()
@@ -829,12 +848,12 @@ def compileTree():
         if not os.path.isfile(os.path.join(source_dir, "__helpers.h")):
             general.sysexit("Error, no previous build directory exists.")
 
-    if Options.isShowProgress() or Options.isShowMemory():
-        general.info(
-            "Total memory usage before running scons: {memory}:".format(
-                memory=MemoryUsage.getHumanReadableProcessMemoryUsage()
-            )
-        )
+    reportMemoryUsage(
+        "before_running_scons",
+        "Total memory usage before running scons"
+        if Options.isShowProgress() or Options.isShowMemory()
+        else None,
+    )
 
     if Options.isShowMemory():
         InstanceCounters.printStats()
@@ -854,7 +873,7 @@ def compileTree():
     general.info("Running C compilation via Scons.")
 
     # Run the Scons to build things.
-    result, options = runSconsBackend(quiet=not Options.isShowScons())
+    result, options = runSconsBackend()
 
     return result, options
 
@@ -885,7 +904,7 @@ of the precise Python interpreter binary and '-m nuitka', e.g. use this
     sys.exit(error_message)
 
 
-def main():
+def _main():
     """Main program flow of Nuitka
 
     At this point, options will be parsed already, Nuitka will be executing
@@ -914,23 +933,25 @@ def main():
         )
     )
 
-    filename = Options.getPositionalArgs()[0]
-
-    # Inform the importing layer about the main script directory, so it can use
-    # it when attempting to follow imports.
-    Importing.setMainScriptDirectory(
-        main_dir=os.path.dirname(os.path.abspath(filename))
+    reportMemoryUsage(
+        "after_launch",
+        "Total memory usage before processing:"
+        if Options.isShowProgress() or Options.isShowMemory()
+        else None,
     )
 
+    _setupFromMainFilenames()
+
     addIncludedDataFilesFromFileOptions()
+    addIncludedDataFilesFromPackageOptions()
 
     # Turn that source code into a node tree structure.
     try:
-        main_module = _createNodeTree(filename=filename)
+        main_module = _createMainModule()
     except (SyntaxError, IndentationError) as e:
         handleSyntaxError(e)
 
-    addIncludedDataFilesFromPackageOptions()
+    addIncludedDataFilesFromPlugins()
 
     dumpTreeXML()
 
@@ -953,7 +974,7 @@ def main():
 
     if Options.shallNotDoExecCCompilerCall():
         if Options.isShowMemory():
-            MemoryUsage.showMemoryTrace()
+            showMemoryTrace()
 
         sys.exit(0)
 
@@ -987,7 +1008,7 @@ def main():
             packDistFolderToOnefile(dist_dir)
 
             if Options.isRemoveBuildDir():
-                general.info("Removing dist folder %r." % dist_dir)
+                general.info("Removing dist folder '%s'." % dist_dir)
 
                 removeDirectory(path=dist_dir, ignore_errors=False)
             else:
@@ -1034,10 +1055,7 @@ not use compiled code while it exists."""
 
     general.info("Successfully created '%s'." % final_filename)
 
-    report_filename = Options.getCompilationReportFilename()
-
-    if report_filename:
-        writeCompilationReport(report_filename)
+    writeCompilationReports(aborted=False)
 
     run_filename = OutputDirectories.getResultRunFilename(
         onefile=Options.isOnefileMode()
@@ -1063,3 +1081,14 @@ not use compiled code while it exists."""
                 "Execute it by launching '%s', the batch file needs to set environment."
                 % run_filename
             )
+
+
+def main():
+    try:
+        _main()
+    finally:
+        if sys.exc_info() != (None, None, None):
+            try:
+                writeCompilationReports(aborted=True)
+            except BaseException as e:  # Catch all the things, pylint: disable=broad-except
+                general.warning("Report writing prevented by exception %s" % e)
