@@ -1,4 +1,4 @@
-#     Copyright 2021, Kay Hayen, mailto:kay.hayen@gmail.com
+#     Copyright 2022, Kay Hayen, mailto:kay.hayen@gmail.com
 #
 #     Part of "Nuitka", an optimizing Python compiler that is compatible and
 #     integrates with CPython, but also works on its own.
@@ -57,8 +57,13 @@ def printSeparator(level=0):
     print("    " * level, "*" * 10)
 
 
-def printLine(*what):
-    print(*what)
+def printLine(*what, **kwargs):
+    is_atty = sys.stdout.isatty()
+
+    if progress and is_atty:
+        progress.close()
+
+    print(*what, **kwargs)
 
 
 def printError(message):
@@ -71,6 +76,8 @@ def flushStandardOutputs():
 
 
 def _getEnableStyleCode(style):
+    style = _aliasStyle(style)
+
     if style == "pink":
         style = "\033[95m"
     elif style == "blue":
@@ -119,7 +126,7 @@ def _getIoctlGWINSZ(fd):
         import fcntl
         import termios
 
-        return struct.unpack('hh', fcntl.ioctl(fd, termios.TIOCGWINSZ, '1234'))
+        return struct.unpack("hh", fcntl.ioctl(fd, termios.TIOCGWINSZ, "1234"))
     except BaseException:  # Catch all the things, pylint: disable=broad-except
         return None
 
@@ -181,13 +188,15 @@ def _getTerminalSize():
             return 1000
 
 
-def aliasStyle(style):
+def _aliasStyle(style):
     if style == "test-prepare":
         return "pink"
     if style == "test-progress":
         return "blue"
     if style == "test-debug":
         return "bold"
+    if style == "link":
+        return "blue"
     else:
         return style
 
@@ -195,9 +204,6 @@ def aliasStyle(style):
 def _my_print(file_output, is_atty, args, kwargs):
     if "style" in kwargs:
         style = kwargs["style"]
-
-        style = aliasStyle(style)
-
         del kwargs["style"]
 
         if "end" in kwargs:
@@ -253,36 +259,47 @@ class OurLogger(object):
         self.base_style = base_style
         self.is_quiet = quiet
 
-        # Can disable warnings, we do that for options parsing during re-execution.
-        self.is_no_warnings = False
-
     def my_print(self, message, **kwargs):
         # For overload, pylint: disable=no-self-use
         my_print(message, **kwargs)
 
-    def warning(self, message, style="red"):
-        if not self.is_no_warnings:
-            if self.name:
-                prefix = "%s:WARNING: " % self.name
-            else:
-                prefix = "WARNING: "
+    def warning(self, message, style="red", mnemonic=None):
+        if mnemonic is not None:
+            from .Options import shallDisplayWarningMnemonic
 
-            style = style or self.base_style
+            if not shallDisplayWarningMnemonic(mnemonic):
+                return
 
-            if sys.stderr.isatty():
-                width = _getTerminalSize()
-            else:
-                width = 10000
+        if self.name:
+            prefix = "%s:WARNING: " % self.name
+        else:
+            prefix = "WARNING: "
 
-            formatted_message = textwrap.fill(
-                message,
-                width=width,
-                initial_indent=prefix,
-                subsequent_indent=prefix,
-                break_on_hyphens=False,
-                expand_tabs=False,
+        style = style or self.base_style
+
+        if sys.stderr.isatty():
+            width = _getTerminalSize() or 10000
+        else:
+            width = 10000
+
+        formatted_message = textwrap.fill(
+            message,
+            width=width,
+            initial_indent=prefix,
+            subsequent_indent=prefix,
+            break_on_hyphens=False,
+            break_long_words=False,
+            expand_tabs=False,
+            replace_whitespace=False,
+        )
+        self.my_print(formatted_message, style=style, file=sys.stderr)
+
+        if mnemonic is not None:
+            self.warning(
+                """    Complex topic! More information can be found at %shttps://nuitka.net/info/%s.html"""
+                % (_getEnableStyleCode("link"), mnemonic),
+                style=style,
             )
-            self.my_print(formatted_message, style=style, file=sys.stderr)
 
     def sysexit(self, message="", exit_code=1):
         from nuitka.Progress import closeProgressBar
@@ -290,7 +307,17 @@ class OurLogger(object):
         closeProgressBar()
 
         if message:
-            self.my_print("FATAL: %s" % message, style="red", file=sys.stderr)
+            if exit_code != 0:
+                self.my_print(
+                    "FATAL: %s" % message,
+                    style="red",
+                    file=sys.stderr,
+                )
+            else:
+                self.my_print(
+                    message,
+                    file=sys.stderr,
+                )
 
         sys.exit(exit_code)
 
@@ -360,14 +387,16 @@ memory_logger = OurLogger("Nuitka-Memory")
 dependencies_logger = OurLogger("Nuitka-Dependencies")
 optimization_logger = FileLogger("Nuitka-Optimization")
 pgo_logger = FileLogger("Nuitka-PGO")
-codegen_logger = OurLogger("Nuitka-Codegen")
+code_generation_logger = OurLogger("Nuitka-CodeGen")
 inclusion_logger = FileLogger("Nuitka-Inclusion")
 scons_logger = OurLogger("Nuitka-Scons")
 scons_details_logger = OurLogger("Nuitka-Scons")
 postprocessing_logger = OurLogger("Nuitka-Postprocessing")
 options_logger = OurLogger("Nuitka-Options")
 unusual_logger = OurLogger("Nuitka-Unusual")
-datacomposer_logger = OurLogger("Nuitka-Datacomposer")
+data_composer_logger = OurLogger("Nuitka-DataComposer")
 onefile_logger = OurLogger("Nuitka-Onefile")
 tools_logger = OurLogger("Nuitka-Tools")
 wheel_logger = OurLogger("Nuitka-Wheel")
+cache_logger = OurLogger("Nuitka-Cache")
+reports_logger = OurLogger("Nuitka-Reports")
