@@ -1,4 +1,4 @@
-#     Copyright 2021, Kay Hayen, mailto:kay.hayen@gmail.com
+#     Copyright 2022, Kay Hayen, mailto:kay.hayen@gmail.com
 #
 #     Part of "Nuitka", an optimizing Python compiler that is compatible and
 #     integrates with CPython, but also works on its own.
@@ -35,14 +35,31 @@ def main():
 
     # Also high complexity.
     # pylint: disable=too-many-branches,too-many-locals,too-many-statements
+    if (
+        os.name == "nt"
+        and os.path.normcase(os.path.basename(sys.executable)) == "pythonw.exe"
+    ):
+        import ctypes
+
+        ctypes.windll.user32.MessageBoxW(
+            None,
+            "You have to use the 'python.exe' and not a 'pythonw.exe' to run Nuitka",
+            "Error",
+            0x1000,  # MB_SYSTEMMODAL
+        )
+        sys.exit(1)
 
     if "NUITKA_BINARY_NAME" in os.environ:
         sys.argv[0] = os.environ["NUITKA_BINARY_NAME"]
 
     if "NUITKA_PYTHONPATH" in os.environ:
         # Restore the PYTHONPATH gained from the site module, that we chose not
-        # to have imported. pylint: disable=eval-used
-        sys.path = eval(os.environ["NUITKA_PYTHONPATH"])
+        # to have imported during compilation. For loading ast module, we need
+        # one element, that is not necessarily in our current path.
+        sys.path = [os.environ["NUITKA_PYTHONPATH_AST"]]
+        import ast
+
+        sys.path = ast.literal_eval(os.environ["NUITKA_PYTHONPATH"])
         del os.environ["NUITKA_PYTHONPATH"]
     else:
         # Remove path element added for being called via "__main__.py", this can
@@ -56,10 +73,10 @@ def main():
 
     # We will run with the Python configuration as specified by the user, if it does
     # not match, we restart ourselves with matching configuration.
-    needs_reexec = False
+    needs_re_execution = False
 
     if sys.flags.no_site == 0:
-        needs_reexec = True
+        needs_re_execution = True
 
     # The hash randomization totally changes the created source code created,
     # changing it every single time Nuitka is run. This kills any attempt at
@@ -67,10 +84,10 @@ def main():
     # actually may still use it, during compilation we don't want to. So lets
     # disable it.
     if os.environ.get("PYTHONHASHSEED", "-1") != "0":
-        needs_reexec = True
+        needs_re_execution = True
 
     # In case we need to re-execute.
-    if needs_reexec:
+    if needs_re_execution and "debugpy" not in sys.modules:
         from nuitka.utils.ReExecute import reExecuteNuitka  # isort:skip
 
         # Does not return
@@ -101,20 +118,17 @@ def main():
 
     if "NUITKA_NAMESPACES" in os.environ:
         # Restore the detected name space packages, that were force loaded in
-        # site.py, and will need a free pass later on. pylint: disable=eval-used
-
+        # site.py, and will need a free pass later on
         from nuitka.importing.PreloadedPackages import setPreloadedPackagePaths
 
-        setPreloadedPackagePaths(eval(os.environ["NUITKA_NAMESPACES"]))
+        setPreloadedPackagePaths(ast.literal_eval(os.environ["NUITKA_NAMESPACES"]))
         del os.environ["NUITKA_NAMESPACES"]
 
     if "NUITKA_PTH_IMPORTED" in os.environ:
         # Restore the packages that the ".pth" files asked to import.
-        # pylint: disable=eval-used
-
         from nuitka.importing.PreloadedPackages import setPthImportedPackages
 
-        setPthImportedPackages(eval(os.environ["NUITKA_PTH_IMPORTED"]))
+        setPthImportedPackages(ast.literal_eval(os.environ["NUITKA_PTH_IMPORTED"]))
         del os.environ["NUITKA_PTH_IMPORTED"]
 
     # Now the real main program of Nuitka can take over.
