@@ -1,4 +1,4 @@
-#     Copyright 2021, Kay Hayen, mailto:kay.hayen@gmail.com
+#     Copyright 2022, Kay Hayen, mailto:kay.hayen@gmail.com
 #
 #     Part of "Nuitka", an optimizing Python compiler that is compatible and
 #     integrates with CPython, but also works on its own.
@@ -32,8 +32,8 @@ from nuitka.Builtins import (
 )
 
 # TODO: Move to constants
-from nuitka.codegen.Namify import namifyConstant
-from nuitka.containers.oset import OrderedSet
+from nuitka.code_generation.Namify import namifyConstant
+from nuitka.containers.OrderedSets import OrderedSet
 from nuitka.PythonVersions import python_version
 from nuitka.utils.FileOperations import openTextFile
 
