@@ -1,4 +1,4 @@
-#     Copyright 2020, Kay Hayen, mailto:kay.hayen@gmail.com
+#     Copyright 2022, Kay Hayen, mailto:kay.hayen@gmail.com
 #
 #     Part of "Nuitka", an optimizing Python compiler that is compatible and
 #     integrates with CPython, but also works on its own.
@@ -21,13 +21,13 @@
     text, explaining things about its context.
 """
 
-from .ExpressionBases import (
-    ExpressionBuiltinSingleArgBase,
-    ExpressionChildrenHavingBase,
-)
+from .ChildrenHavingMixins import ChildrenHavingIteratorDefaultMixin
+from .ExpressionBases import ExpressionBase, ExpressionBuiltinSingleArgBase
 
 
 class ExpressionBuiltinNext1(ExpressionBuiltinSingleArgBase):
+    __slots__ = ("may_raise",)
+
     kind = "EXPRESSION_BUILTIN_NEXT1"
 
     def __init__(self, value, source_ref):
@@ -35,26 +35,25 @@ class ExpressionBuiltinNext1(ExpressionBuiltinSingleArgBase):
             self, value=value, source_ref=source_ref
         )
 
-        self.may_not_raise = False
+        self.may_raise = True
 
     def computeExpression(self, trace_collection):
-        self.may_not_raise, result = self.subnode_value.computeExpressionNext1(
+        self.may_raise, result = self.subnode_value.computeExpressionNext1(
             next_node=self, trace_collection=trace_collection
         )
 
         return result
 
     def mayRaiseExceptionOperation(self):
-        return not self.may_not_raise
+        return self.may_raise
 
     def mayRaiseException(self, exception_type):
-        return (
-            self.subnode_value.mayRaiseException(exception_type)
-            or self.mayRaiseExceptionOperation()
-        )
+        return self.may_raise or self.subnode_value.mayRaiseException(exception_type)
 
 
 class ExpressionSpecialUnpack(ExpressionBuiltinNext1):
+    __slots__ = ("count", "expected", "starred")
+
     kind = "EXPRESSION_SPECIAL_UNPACK"
 
     def __init__(self, value, count, expected, starred, source_ref):
@@ -62,7 +61,7 @@ class ExpressionSpecialUnpack(ExpressionBuiltinNext1):
 
         self.count = int(count)
 
-        # TODO: Unused before 3.5 or higher, maybe specialize for it.
+        # TODO: Unused before 3.5 or higher, and even then starred is rare, maybe specialize for it.
         self.expected = int(expected)
         self.starred = starred
 
@@ -83,20 +82,20 @@ class ExpressionSpecialUnpack(ExpressionBuiltinNext1):
         return self.starred
 
 
-class ExpressionBuiltinNext2(ExpressionChildrenHavingBase):
+class ExpressionBuiltinNext2(ChildrenHavingIteratorDefaultMixin, ExpressionBase):
     kind = "EXPRESSION_BUILTIN_NEXT2"
 
     named_children = ("iterator", "default")
-    getIterator = ExpressionChildrenHavingBase.childGetter("iterator")
-    getDefault = ExpressionChildrenHavingBase.childGetter("default")
 
     def __init__(self, iterator, default, source_ref):
-        ExpressionChildrenHavingBase.__init__(
+        ChildrenHavingIteratorDefaultMixin.__init__(
             self,
-            values={"iterator": iterator, "default": default},
-            source_ref=source_ref,
+            iterator=iterator,
+            default=default,
         )
 
+        ExpressionBase.__init__(self, source_ref)
+
     def computeExpression(self, trace_collection):
         # TODO: The "iterator" should be investigated here, if it is iterable,
         # or if the default is raising.
