@@ -1,4 +1,4 @@
-#     Copyright 2021, Kay Hayen, mailto:kay.hayen@gmail.com
+#     Copyright 2022, Kay Hayen, mailto:kay.hayen@gmail.com
 #
 #     Part of "Nuitka", an optimizing Python compiler that is compatible and
 #     integrates with CPython, but also works on its own.
@@ -23,12 +23,17 @@ that should allow some important optimizations.
 
 from nuitka.specs import BuiltinParameterSpecs
 
+from .ChildrenHavingMixins import (
+    ChildHavingValueMixin,
+    ChildrenExpressionBuiltinBytearray3Mixin,
+    ChildrenExpressionTypeOperationPrepareMixin,
+    ChildrenHavingValueOptionalEncodingOptionalErrorsOptionalMixin,
+)
 from .ConstantRefNodes import makeConstantRefNode
 from .ExpressionBases import (
     CompileTimeConstantExpressionBase,
+    ExpressionBase,
     ExpressionBuiltinSingleArgBase,
-    ExpressionChildHavingBase,
-    ExpressionChildrenHavingBase,
     ExpressionSpecBasedComputationMixin,
 )
 from .ExpressionShapeMixins import (
@@ -38,6 +43,8 @@ from .ExpressionShapeMixins import (
     ExpressionFrozensetShapeExactMixin,
     ExpressionListShapeExactMixin,
     ExpressionSetShapeExactMixin,
+    ExpressionStrDerivedShapeMixin,
+    ExpressionStrOrUnicodeDerivedShapeMixin,
     ExpressionTupleShapeExactMixin,
 )
 from .NodeMakingHelpers import (
@@ -57,15 +64,17 @@ class ExpressionBuiltinTypeBase(ExpressionBuiltinSingleArgBase):
 
 
 class ExpressionBuiltinContainerBase(
-    ExpressionSpecBasedComputationMixin, ExpressionChildHavingBase
+    ExpressionSpecBasedComputationMixin, ChildHavingValueMixin, ExpressionBase
 ):
-
+    # For overload.
     builtin_spec = None
 
-    named_child = "value"
+    named_children = ("value",)
 
     def __init__(self, value, source_ref):
-        ExpressionChildHavingBase.__init__(self, value=value, source_ref=source_ref)
+        ChildHavingValueMixin.__init__(self, value=value)
+
+        ExpressionBase.__init__(self, source_ref)
 
     def computeExpression(self, trace_collection):
         value = self.subnode_value
@@ -82,6 +91,9 @@ class ExpressionBuiltinContainerBase(
             else:
                 return self, None, None
         else:
+            # They take over the variable content, exposing it to currently untraced usages.
+            value.onContentEscapes(trace_collection)
+
             return self.computeBuiltinSpec(
                 trace_collection=trace_collection, given_values=(value,)
             )
@@ -119,13 +131,15 @@ class ExpressionBuiltinFrozenset(
     builtin_spec = BuiltinParameterSpecs.builtin_frozenset_spec
 
 
-class ExpressionBuiltinFloat(ExpressionChildHavingBase):
+class ExpressionBuiltinFloat(ChildHavingValueMixin, ExpressionBase):
     kind = "EXPRESSION_BUILTIN_FLOAT"
 
-    named_child = "value"
+    named_children = ("value",)
 
     def __init__(self, value, source_ref):
-        ExpressionChildHavingBase.__init__(self, value=value, source_ref=source_ref)
+        ChildHavingValueMixin.__init__(self, value=value)
+
+        ExpressionBase.__init__(self, source_ref)
 
     @staticmethod
     def getTypeShape():
@@ -165,21 +179,38 @@ class ExpressionBuiltinBool(ExpressionBoolShapeExactMixin, ExpressionBuiltinType
                 "Predicted truth value of built-in bool argument",
             )
 
+        if value.hasShapeBoolExact():
+            return (
+                value,
+                "new_expression",
+                "Eliminated boolean conversion of boolean value.",
+            )
+
         return ExpressionBuiltinTypeBase.computeExpression(self, trace_collection)
 
+    def mayRaiseException(self, exception_type):
+        return self.subnode_value.mayRaiseException(
+            exception_type
+        ) or self.subnode_value.mayRaiseExceptionBool(exception_type)
+
 
 class ExpressionBuiltinUnicodeBase(
-    ExpressionSpecBasedComputationMixin, ExpressionChildrenHavingBase
+    ExpressionSpecBasedComputationMixin,
+    ChildrenHavingValueOptionalEncodingOptionalErrorsOptionalMixin,
+    ExpressionBase,
 ):
-    named_children = ("value", "encoding", "errors")
+    named_children = ("value|optional", "encoding|optional", "errors|optional")
 
     def __init__(self, value, encoding, errors, source_ref):
-        ExpressionChildrenHavingBase.__init__(
+        ChildrenHavingValueOptionalEncodingOptionalErrorsOptionalMixin.__init__(
             self,
-            values={"value": value, "encoding": encoding, "errors": errors},
-            source_ref=source_ref,
+            value=value,
+            encoding=encoding,
+            errors=errors,
         )
 
+        ExpressionBase.__init__(self, source_ref)
+
     def computeExpression(self, trace_collection):
         args = [self.subnode_value, self.subnode_encoding, self.subnode_errors]
 
@@ -198,7 +229,9 @@ class ExpressionBuiltinUnicodeBase(
         )
 
 
-class ExpressionBuiltinStrP2(ExpressionBuiltinTypeBase):
+class ExpressionBuiltinStrP2(
+    ExpressionStrOrUnicodeDerivedShapeMixin, ExpressionBuiltinTypeBase
+):
     """Python2 built-in str call."""
 
     kind = "EXPRESSION_BUILTIN_STR_P2"
@@ -242,7 +275,9 @@ class ExpressionBuiltinUnicodeP2(ExpressionBuiltinUnicodeBase):
         return tshape_unicode_derived
 
 
-class ExpressionBuiltinStrP3(ExpressionBuiltinUnicodeBase):
+class ExpressionBuiltinStrP3(
+    ExpressionStrDerivedShapeMixin, ExpressionBuiltinUnicodeBase
+):
     """Python3 built-in str call."""
 
     kind = "EXPRESSION_BUILTIN_STR_P3"
@@ -262,13 +297,15 @@ class ExpressionBuiltinBytes3(
     builtin_spec = BuiltinParameterSpecs.builtin_bytes_p3_spec
 
 
-class ExpressionBuiltinBytes1(ExpressionChildHavingBase):
+class ExpressionBuiltinBytes1(ChildHavingValueMixin, ExpressionBase):
     kind = "EXPRESSION_BUILTIN_BYTES1"
 
-    named_child = "value"
+    named_children = ("value",)
 
     def __init__(self, value, source_ref):
-        ExpressionChildHavingBase.__init__(self, value=value, source_ref=source_ref)
+        ChildHavingValueMixin.__init__(self, value=value)
+
+        ExpressionBase.__init__(self, source_ref)
 
     @staticmethod
     def getTypeShape():
@@ -296,21 +333,26 @@ class ExpressionBuiltinBytearray1(
 
 
 class ExpressionBuiltinBytearray3(
-    ExpressionBytearrayShapeExactMixin, ExpressionChildrenHavingBase
+    ExpressionBytearrayShapeExactMixin,
+    ChildrenExpressionBuiltinBytearray3Mixin,
+    ExpressionBase,
 ):
     kind = "EXPRESSION_BUILTIN_BYTEARRAY3"
 
-    named_children = ("string", "encoding", "errors")
+    named_children = ("string", "encoding|optional", "errors|optional")
 
     builtin_spec = BuiltinParameterSpecs.builtin_bytearray_spec
 
     def __init__(self, string, encoding, errors, source_ref):
-        ExpressionChildrenHavingBase.__init__(
+        ChildrenExpressionBuiltinBytearray3Mixin.__init__(
             self,
-            values={"string": string, "encoding": encoding, "errors": errors},
-            source_ref=source_ref,
+            string=string,
+            encoding=encoding,
+            errors=errors,
         )
 
+        ExpressionBase.__init__(self, source_ref)
+
     def computeExpression(self, trace_collection):
         trace_collection.onExceptionRaiseExit(BaseException)
 
@@ -323,7 +365,7 @@ class ExpressionConstantGenericAlias(CompileTimeConstantExpressionBase):
     __slots__ = ("generic_alias",)
 
     def __init__(self, generic_alias, source_ref):
-        CompileTimeConstantExpressionBase.__init__(self, source_ref=source_ref)
+        CompileTimeConstantExpressionBase.__init__(self, source_ref)
 
         self.generic_alias = generic_alias
 
@@ -346,3 +388,63 @@ class ExpressionConstantGenericAlias(CompileTimeConstantExpressionBase):
     def computeExpressionRaw(self, trace_collection):
         # Nothing much to do.
         return self, None, None
+
+
+class ExpressionConstantUnionType(CompileTimeConstantExpressionBase):
+    kind = "EXPRESSION_CONSTANT_UNION_TYPE"
+
+    __slots__ = ("union_type",)
+
+    def __init__(self, union_type, source_ref):
+        CompileTimeConstantExpressionBase.__init__(self, source_ref)
+
+        self.union_type = union_type
+
+    def finalize(self):
+        del self.parent
+
+    def getDetails(self):
+        return {"union_type": self.union_type}
+
+    def getCompileTimeConstant(self):
+        return self.union_type
+
+    def getStrValue(self):
+        return makeConstantRefNode(
+            constant=str(self.getCompileTimeConstant()),
+            user_provided=True,
+            source_ref=self.source_ref,
+        )
+
+    def computeExpressionRaw(self, trace_collection):
+        # Nothing much to do.
+        return self, None, None
+
+
+class ExpressionTypeOperationPrepare(
+    ChildrenExpressionTypeOperationPrepareMixin, ExpressionBase
+):
+    kind = "EXPRESSION_TYPE_OPERATION_PREPARE"
+
+    named_children = ("type_arg", "args|optional", "kwargs|optional")
+
+    def __init__(self, type_arg, args, kwargs, source_ref):
+        ChildrenExpressionTypeOperationPrepareMixin.__init__(
+            self, type_arg=type_arg, args=args, kwargs=kwargs
+        )
+
+        ExpressionBase.__init__(self, source_ref)
+
+    def computeExpression(self, trace_collection):
+        if self.subnode_type_arg.isExpressionConstantTypeTypeRef():
+            result = makeConstantReplacementNode(
+                constant={}, node=self, user_provided=False
+            )
+
+            return (
+                result,
+                "new_constant",
+                "Predicted result 'type.__prepare__' as empty dict.",
+            )
+
+        return self, None, None
