@@ -1,4 +1,4 @@
-#     Copyright 2020, Kay Hayen, mailto:kay.hayen@gmail.com
+#     Copyright 2022, Kay Hayen, mailto:kay.hayen@gmail.com
 #
 #     Part of "Nuitka", an optimizing Python compiler that is compatible and
 #     integrates with CPython, but also works on its own.
@@ -24,7 +24,11 @@ types only.
 
 
 def checkStatementsSequenceOrNone(value):
-    assert value is None or value.kind == "STATEMENTS_SEQUENCE", value
+    if value is not None:
+        assert value.kind == "STATEMENTS_SEQUENCE", value
+
+        if not value.subnode_statements:
+            return None
 
     return value
 
@@ -33,3 +37,17 @@ def checkStatementsSequence(value):
     assert value is not None and value.kind == "STATEMENTS_SEQUENCE", value
 
     return value
+
+
+def convertNoneConstantToNone(node):
+    if node is None or node.isExpressionConstantNoneRef():
+        return None
+    else:
+        return node
+
+
+def convertEmptyStrConstantToNone(node):
+    if node is None or node.isExpressionConstantStrEmptyRef():
+        return None
+    else:
+        return node
