@@ -1,4 +1,4 @@
-#     Copyright 2021, Kay Hayen, mailto:kay.hayen@gmail.com
+#     Copyright 2022, Kay Hayen, mailto:kay.hayen@gmail.com
 #
 #     Part of "Nuitka", an optimizing Python compiler that is compatible and
 #     integrates with CPython, but also works on its own.
@@ -24,28 +24,25 @@ together and cross-module optimizations are the most difficult to tackle.
 import os
 
 from nuitka import Options, Variables
-from nuitka.containers.oset import OrderedSet
-from nuitka.importing.Importing import (
-    getModuleNameAndKindFromFilename,
-    locateModule,
-)
+from nuitka.containers.OrderedSets import OrderedSet
+from nuitka.importing.Importing import locateModule
 from nuitka.importing.Recursion import decideRecursion, recurseTo
 from nuitka.ModuleRegistry import getModuleByName, getOwnerFromCodeName
 from nuitka.optimizations.TraceCollections import TraceCollectionModule
 from nuitka.PythonVersions import python_version
 from nuitka.SourceCodeReferences import fromFilename
-from nuitka.tree.Operations import DetectUsedModules, visitTree
-from nuitka.tree.SourceReading import readSourceCodeFromFilename
+from nuitka.tree.SourceHandling import readSourceCodeFromFilename
 from nuitka.utils.CStrings import encodePythonIdentifierToC
 from nuitka.utils.FileOperations import getFileContentByLine
 from nuitka.utils.ModuleNames import ModuleName
 
-from .Checkers import checkStatementsSequenceOrNone
+from .ChildrenHavingMixins import (
+    ModuleChildrenHavingBodyOptionalStatementsOrNoneFunctionsTupleMixin,
+)
 from .FutureSpecs import fromFlags
 from .IndicatorMixins import EntryPointMixin, MarkNeedsAnnotationsMixin
 from .LocalsScopes import getLocalsDictHandle
 from .NodeBases import (
-    ChildrenHavingMixin,
     ClosureGiverNodeMixin,
     NodeBase,
     extractKindAndArgsFromXML,
@@ -90,7 +87,12 @@ class PythonModuleBase(NodeBase):
         package = getModuleByName(package_name)
 
         if package_name is not None and package is None:
-            _package_name, package_filename, finding = locateModule(
+            (
+                _package_name,
+                package_filename,
+                package_module_kind,
+                finding,
+            ) = locateModule(
                 module_name=package_name,
                 parent_package=None,
                 level=0,
@@ -106,15 +108,12 @@ class PythonModuleBase(NodeBase):
 
             assert package_filename is not None, (package_name, finding)
 
-            _package_name, package_kind = getModuleNameAndKindFromFilename(
-                package_filename
-            )
             # assert _package_name == self.package_name, (package_filename, _package_name, self.package_name)
 
             decision, _reason = decideRecursion(
                 module_filename=package_filename,
                 module_name=package_name,
-                module_kind=package_kind,
+                module_kind=package_module_kind,
             )
 
             if decision is not None:
@@ -124,7 +123,9 @@ class PythonModuleBase(NodeBase):
                     else None,
                     module_name=package_name,
                     module_filename=package_filename,
-                    module_kind="py",
+                    module_kind=package_module_kind,
+                    using_module=self,
+                    source_ref=self.source_ref,
                     reason="Containing package of '%s'." % self.getFullName(),
                 )
 
@@ -200,9 +201,17 @@ class PythonModuleBase(NodeBase):
 
             return result
 
+    @staticmethod
+    def isStatement():
+        return False
+
+    @staticmethod
+    def isExpression():
+        return False
+
 
 class CompiledPythonModule(
-    ChildrenHavingMixin,
+    ModuleChildrenHavingBodyOptionalStatementsOrNoneFunctionsTupleMixin,
     ClosureGiverNodeMixin,
     MarkNeedsAnnotationsMixin,
     EntryPointMixin,
@@ -237,9 +246,7 @@ class CompiledPythonModule(
         "locals_scope",
     )
 
-    named_children = ("body", "functions")
-
-    checkers = {"body": checkStatementsSequenceOrNone}
+    named_children = ("body|statements_or_none+setter", "functions|tuple+setter")
 
     def __init__(self, module_name, is_top, mode, future_spec, source_ref):
         PythonModuleBase.__init__(self, module_name=module_name, source_ref=source_ref)
@@ -248,8 +255,10 @@ class CompiledPythonModule(
             self, name=module_name.getBasename(), code_prefix="module"
         )
 
-        ChildrenHavingMixin.__init__(
-            self, values={"body": None, "functions": ()}  # delayed
+        ModuleChildrenHavingBodyOptionalStatementsOrNoneFunctionsTupleMixin.__init__(
+            self,
+            body=None,  # delayed
+            functions=(),
         )
 
         MarkNeedsAnnotationsMixin.__init__(self)
@@ -384,6 +393,7 @@ class CompiledPythonModule(
         if self.source_code is not None:
             return self.source_code
         else:
+            # This should of course give same result as before.
             return readSourceCodeFromFilename(
                 module_name=self.getFullName(),
                 source_filename=self.getCompileTimeFilename(),
@@ -445,11 +455,15 @@ class CompiledPythonModule(
 
         return encodePythonIdentifierToC(self.getFullName())
 
+    @staticmethod
+    def getChildQualname(function_name):
+        return function_name
+
     def addFunction(self, function_body):
         functions = self.subnode_functions
         assert function_body not in functions
         functions += (function_body,)
-        self.setChild("functions", functions)
+        self.setChildFunctions(functions)
 
     def startTraversal(self):
         self.used_modules = None
@@ -457,21 +471,17 @@ class CompiledPythonModule(
 
     def restartTraversal(self):
         self.visited_functions = set()
+        self.used_modules = None
 
     def getUsedModules(self):
-        if self.used_modules is None:
-            visitor = DetectUsedModules()
-            visitTree(tree=self, visitor=visitor)
-            self.used_modules = visitor.getUsedModules()
-
-        return self.used_modules
+        return self.trace_collection.getModuleUsageAttempts()
 
     def addUsedFunction(self, function_body):
         assert function_body in self.subnode_functions, function_body
 
         assert (
             function_body.isExpressionFunctionBody()
-            or function_body.isExpressionClassBody()
+            or function_body.isExpressionClassBodyBase()
             or function_body.isExpressionGeneratorObjectBody()
             or function_body.isExpressionCoroutineObjectBody()
             or function_body.isExpressionAsyncgenObjectBody()
@@ -538,7 +548,7 @@ class CompiledPythonModule(
             )
 
             if result is not module_body:
-                self.setChild("body", result)
+                self.setChildBody(result)
 
         self.attemptRecursion()
 
@@ -573,13 +583,11 @@ class CompiledPythonModule(
         was_complete = not self.locals_scope.complete
 
         def markAsComplete(body, trace_collection):
-            if (
-                body.locals_scope is not None
-                and body.locals_scope.isMarkedForPropagation()
-            ):
-                body.locals_scope = None
-
             if body.locals_scope is not None:
+                # Make sure the propagated stuff releases memory.
+                if body.locals_scope.isMarkedForPropagation():
+                    body.locals_scope.onPropagationComplete()
+
                 body.locals_scope.markAsComplete(trace_collection)
 
         def markEntryPointAsComplete(body):
@@ -676,6 +684,8 @@ class CompiledPythonModule(
     def getRuntimeNameValue(self):
         if self.isMainModule() and Options.hasPythonFlagPackageMode():
             return "__main__"
+        elif self.module_name.isMultidistModuleName():
+            return "__main__"
         else:
             return self.getFullName().asString()
 
@@ -778,7 +788,8 @@ class UncompiledPythonModule(PythonModuleBase):
     def setUsedModules(self, used_modules):
         self.used_modules = used_modules
 
-    def startTraversal(self):
+    @staticmethod
+    def startTraversal():
         pass
 
 
@@ -859,15 +870,14 @@ class PythonMainModule(CompiledPythonModule):
             function_work.append((function, iter(iter(xml).next()).next()))
 
         for function, xml in function_work:
-            function.setChild(
-                "body",
+            function.setChildBody(
                 fromXML(
                     provider=function, xml=xml, source_ref=function.getSourceReference()
-                ),
+                )
             )
 
-        result.setChild(
-            "body", fromXML(provider=result, xml=args["body"][0], source_ref=source_ref)
+        result.setChildBody(
+            fromXML(provider=result, xml=args["body"][0], source_ref=source_ref)
         )
 
         return result
@@ -942,7 +952,8 @@ class PythonExtensionModule(PythonModuleBase):
     def getFilename(self):
         return self.source_ref.getFilename()
 
-    def startTraversal(self):
+    @staticmethod
+    def startTraversal():
         pass
 
     def isTechnical(self):
@@ -958,7 +969,7 @@ class PythonExtensionModule(PythonModuleBase):
 
         return os.path.join(dirname, filename.split(".")[0]) + ".pyi"
 
-    def _readPyPIFile(self):
+    def _readPyIFile(self):
         """Read the .pyi file if present and scan for dependencies."""
 
         # Complex stuff, pylint: disable=too-many-branches,too-many-statements
@@ -1060,16 +1071,22 @@ class PythonExtensionModule(PythonModuleBase):
                 if self.getFullName().getPackageName() in pyi_deps:
                     pyi_deps.discard(self.getFullName().getPackageName())
 
-                self.used_modules = tuple((pyi_dep, None) for pyi_dep in pyi_deps)
+                self.used_modules = tuple(pyi_deps)
             else:
                 self.used_modules = ()
 
-    def getUsedModules(self):
-        self._readPyPIFile()
+    def getPyIModuleImportedNames(self):
+        self._readPyIFile()
 
         assert "." not in self.used_modules, self
 
         return self.used_modules
 
+    @staticmethod
+    def getUsedModules():
+        # The PyI contents is currently delivered via implicit imports
+        # plugin.
+        return ()
+
     def getParentModule(self):
         return self
