@@ -1,4 +1,4 @@
-#     Copyright 2021, Kay Hayen, mailto:kay.hayen@gmail.com
+#     Copyright 2022, Kay Hayen, mailto:kay.hayen@gmail.com
 #
 #     Part of "Nuitka", an optimizing Python compiler that is compatible and
 #     integrates with CPython, but also works on its own.
@@ -25,7 +25,8 @@ of checks, and add methods automatically.
 from abc import ABCMeta
 
 from nuitka.__past__ import intern
-from nuitka.Errors import NuitkaNodeDesignError
+from nuitka.Errors import NuitkaAssumptionError, NuitkaNodeDesignError
+from nuitka.PythonVersions import python_version
 
 
 def _checkBases(name, bases):
@@ -52,39 +53,52 @@ def _checkBases(name, bases):
 class NodeCheckMetaClass(ABCMeta):
     kinds = {}
 
-    # This is in conflict with Pylint 1.9.2 used for Python2, it
-    # should be "mcls" for one and "cls" for the other.
-    # pylint: disable=I0021,bad-mcs-classmethod-argument
-
     def __new__(cls, name, bases, dictionary):  # pylint: disable=I0021,arguments-differ
         _checkBases(name, bases)
 
         if "__slots__" not in dictionary:
             dictionary["__slots__"] = ()
 
-        if "named_child" in dictionary:
-            named_child = dictionary["named_child"]
-            if type(named_child) is not str:
-                raise NuitkaNodeDesignError(
-                    name,
-                    "Class named_child attribute must be string not",
-                    type(named_child),
-                )
-
-            dictionary["__slots__"] += (intern("subnode_" + dictionary["named_child"]),)
-
         if "named_children" in dictionary:
-            if len(dictionary["named_children"]) <= 1:
-                raise NuitkaNodeDesignError(
-                    name, "Use ExpressionChildHaving for one child node classes"
-                )
-
             assert type(dictionary["named_children"]) is tuple
+
             dictionary["__slots__"] += tuple(
-                intern("subnode_" + named_child)
+                intern("subnode_" + named_child.split("|", 1)[0])
                 for named_child in dictionary["named_children"]
             )
 
+        if "nice_children" in dictionary:
+            assert type(dictionary["nice_children"]) is tuple
+            assert len(dictionary["nice_children"]) == len(dictionary["named_children"])
+
+            dictionary["nice_children_dict"] = dict(
+                (intern(named_child.split("|", 1)[0]), nice_name)
+                for (named_child, nice_name) in zip(
+                    dictionary["named_children"], dictionary["nice_children"]
+                )
+            )
+
+        if "node_attributes" in dictionary:
+            dictionary["__slots__"] += dictionary["node_attributes"]
+
+        assert len(dictionary["__slots__"]) == len(
+            set(dictionary["__slots__"])
+        ), dictionary["__slots__"]
+
+        if "python_version_spec" in dictionary:
+            condition = "%s %s" % (
+                hex(python_version),
+                dictionary["python_version_spec"],
+            )
+
+            # We trust our node class files, pylint: disable=eval-used
+            if not eval(condition):
+
+                def __init__(self, *args, **kwargs):
+                    raise NuitkaAssumptionError(name, "assumption violated", condition)
+
+                dictionary["__init__"] = __init__
+
         # Not a method:
         if "checker" in dictionary:
             dictionary["checker"] = staticmethod(dictionary["checker"])
@@ -92,9 +106,9 @@ class NodeCheckMetaClass(ABCMeta):
         # false alarm, pylint: disable=I0021,too-many-function-args
         return ABCMeta.__new__(cls, name, bases, dictionary)
 
-    def __init__(cls, name, bases, dictionary):  # @NoSelf
+    def __init__(cls, name, bases, dictionary):
 
-        if not name.endswith("Base"):
+        if not name.endswith(("Base", "Mixin")):
             if "kind" not in dictionary:
                 raise NuitkaNodeDesignError(name, "Must provide class variable 'kind'")
 
