@@ -1,4 +1,4 @@
-#     Copyright 2021, Kay Hayen, mailto:kay.hayen@gmail.com
+#     Copyright 2022, Kay Hayen, mailto:kay.hayen@gmail.com
 #
 #     Part of "Nuitka", an optimizing Python compiler that is compatible and
 #     integrates with CPython, but also works on its own.
@@ -17,12 +17,13 @@
 #
 """ Common helper functions for specializing code."""
 
-from nuitka.tools.quality.autoformat.Autoformat import (  # For import from here, pylint: disable=unused-import
-    withFileOpenedAndAutoformatted,
+from nuitka.Constants import the_empty_unicode
+from nuitka.tools.quality.auto_format.AutoFormat import (  # For import from here, pylint: disable=unused-import
+    withFileOpenedAndAutoFormatted,
 )
 
 
-def writeline(output, *args):
+def writeLine(output, *args):
     if not args:
         output.write("\n")
     elif len(args) == 1:
@@ -32,25 +33,25 @@ def writeline(output, *args):
 
 
 # Python2 dict methods:
-python2_dict_methods = (
-    "clear",  # has full dict coverage
-    "copy",  # has full dict coverage
+python2_dict_methods = (  # we have full coverage for all methods
+    "clear",
+    "copy",
     "fromkeys",
-    "get",  # has full dict coverage
-    "has_key",  # has full dict coverage
-    "items",  # has full dict coverage
-    "iteritems",  # has full dict coverage
-    "iterkeys",  # has full dict coverage
-    "itervalues",  # has full dict coverage
-    "keys",  # has full dict coverage
-    "pop",  # has full dict coverage
-    "popitem",  # has full dict coverage
-    "setdefault",  # has full dict coverage
-    "update",  # has full dict coverage
-    "values",  # has full dict coverage
-    "viewitems",  # has full dict coverage
-    "viewkeys",  # has full dict coverage
-    "viewvalues",  # has full dict coverage
+    "get",
+    "has_key",
+    "items",
+    "iteritems",
+    "iterkeys",
+    "itervalues",
+    "keys",
+    "pop",
+    "popitem",
+    "setdefault",
+    "update",
+    "values",
+    "viewitems",
+    "viewkeys",
+    "viewvalues",
 )
 
 python3_dict_methods = (
@@ -68,48 +69,48 @@ python3_dict_methods = (
     "values",
 )
 
-python2_str_methods = (
-    "capitalize",  # has full str coverage
+python2_str_methods = (  # we have full coverage for all methods
+    "capitalize",
     "center",
     "count",
     "decode",
     "encode",
-    "endswith",  # has full str coverage
+    "endswith",
     "expandtabs",
-    "find",  # has full str coverage
+    "find",
     "format",
-    "index",  # has full str coverage
-    "isalnum",  # has full str coverage
-    "isalpha",  # has full str coverage
-    "isdigit",  # has full str coverage
-    "islower",  # has full str coverage
-    "isspace",  # has full str coverage
-    "istitle",  # has full str coverage
-    "isupper",  # has full str coverage
-    "join",  # has full str coverage
+    "index",
+    "isalnum",
+    "isalpha",
+    "isdigit",
+    "islower",
+    "isspace",
+    "istitle",
+    "isupper",
+    "join",
     "ljust",
-    "lower",  # has full str coverage
-    "lstrip",  # has full str coverage
-    "partition",  # has full str coverage
+    "lower",
+    "lstrip",
+    "partition",
     "replace",
-    "rfind",  # has full str coverage
-    "rindex",  # has full str coverage
+    "rfind",
+    "rindex",
     "rjust",
-    "rpartition",  # has full str coverage
-    "rsplit",  # has full str coverage
-    "rstrip",  # has full str coverage
-    "split",  # has full str coverage
+    "rpartition",
+    "rsplit",
+    "rstrip",
+    "split",
     "splitlines",
-    "startswith",  # has full str coverage
-    "strip",  # has full str coverage
-    "swapcase",  # has full str coverage
-    "title",  # has full str coverage
+    "startswith",
+    "strip",
+    "swapcase",
+    "title",
     "translate",
-    "upper",  # has full str coverage
+    "upper",
     "zfill",
 )
 
-python3_str_methods = (
+python3_str_methods = (  # we have full coverage for all methods
     "capitalize",
     "casefold",
     "center",
@@ -213,21 +214,25 @@ python3_bytes_methods = (
     "find",
     # static method
     # "fromhex",
-    "hex",
+    # TODO: Python3.5 or higher:
+    # "hex",
     "index",
     "isalnum",
     "isalpha",
-    "isascii",
     "isdigit",
     "islower",
     "isspace",
     "istitle",
     "isupper",
+    # TODO: Python3.7 or higher:
+    # "ishex",
+    # "isascii",
     "join",
     "ljust",
     "lower",
     "lstrip",
-    "maketrans",
+    # static method
+    # "maketrans",
     "partition",
     # TODO: Python3.9 or higher:
     # "removeprefix",
@@ -250,6 +255,41 @@ python3_bytes_methods = (
     "zfill",
 )
 
+python2_list_methods = (
+    "append",
+    "count",
+    "extend",
+    "index",
+    "insert",
+    "pop",
+    "remove",
+    "reverse",
+    "sort",
+)
+
+python3_list_methods = (
+    "append",
+    "clear",
+    "copy",
+    "count",
+    "extend",
+    "index",
+    "insert",
+    "pop",
+    "remove",
+    "reverse",
+    "sort",
+)
+
+python2_type_methods = (
+    # TODO: Totally incomplete
+)
+
+python3_type_methods = (
+    # TODO: Totally incomplete
+    "__prepare__",
+)
+
 
 def getMethodVariations(spec_module, shape_name, method_name, must_exist=False):
     spec_name = shape_name.split("_")[-1] + "_" + method_name + "_spec"
@@ -270,31 +310,50 @@ def getMethodVariations(spec_module, shape_name, method_name, must_exist=False):
                 spec.getStarListArgumentName(),
                 spec.getStarDictArgumentName(),
             )
+
             arg_name_mapping = {
-                "list_args": "iterable",
-                "kw_args": "pairs",
+                "list_args": spec.getStarListArgumentName(),
             }
         else:
             required = spec.getArgumentCount() - spec.getDefaultCount()
 
             arg_counts = tuple(range(required, spec.getArgumentCount() + 1))
 
-            arg_names = spec.getArgumentNames()
+            arg_names = spec.getParameterNames()
             arg_name_mapping = {}
+
+        arg_tests = [
+            (
+                ""
+                if arg_name
+                in (spec.getStarListArgumentName(), spec.getStarDictArgumentName())
+                else "is not None"
+            )
+            for arg_name in arg_names
+        ]
+
+        result_shape = spec.getTypeShape()
+
     else:
-        arg_names = arg_name_mapping = arg_counts = None
+        arg_names = arg_name_mapping = arg_counts = arg_tests = result_shape = None
 
-    return present, arg_names, arg_name_mapping, arg_counts
+    return present, arg_names, arg_tests, arg_name_mapping, arg_counts, result_shape
 
 
-def formatArgs(args, starting=True, finishing=True):
+def getSpecs(spec_module):
+    for name in sorted(dir(spec_module)):
+        if name.endswith("_spec"):
+            yield name, getattr(spec_module, name)
+
+
+def formatArgs(args, starting=True, finishing=True, quoted=False):
     result = []
     if args:
         if not starting:
             result.append(",")
 
         for arg in args:
-            result.append(arg)
+            result.append('"%s"' % arg if quoted else arg)
 
             if arg is not args[-1] or not finishing:
                 result.append(",")
@@ -307,7 +366,7 @@ def check():
         for method_name in python2_str_methods:
             assert hasattr("", method_name), method_name
         for method_name in python2_unicode_methods:
-            assert hasattr(u"", method_name), method_name
+            assert hasattr(the_empty_unicode, method_name), method_name
         for method_name in python2_dict_methods:
             assert hasattr({}, method_name), method_name
     else:
