@@ -1,4 +1,4 @@
-#     Copyright 2021, Kay Hayen, mailto:kay.hayen@gmail.com
+#     Copyright 2022, Kay Hayen, mailto:kay.hayen@gmail.com
 #
 #     Part of "Nuitka", an optimizing Python compiler that is compatible and
 #     integrates with CPython, but also works on its own.
@@ -50,7 +50,6 @@ catching and passing in exceptions raised.
 """
 import marshal
 import os
-import sys
 
 from nuitka import (
     ModuleRegistry,
@@ -59,17 +58,17 @@ from nuitka import (
     SourceCodeReferences,
 )
 from nuitka.__past__ import long, unicode
-from nuitka.Caching import (
-    getCachedImportedModulesNames,
-    hasCachedImportedModulesNames,
+from nuitka.BytecodeCaching import (
+    getCachedImportedModuleUsageAttempts,
+    hasCachedImportedModuleUsageAttempts,
 )
-from nuitka.containers.oset import OrderedSet
+from nuitka.containers.OrderedSets import OrderedSet
 from nuitka.Errors import CodeTooComplexCode
 from nuitka.freezer.Standalone import detectEarlyImports
 from nuitka.importing import Importing
 from nuitka.importing.ImportCache import addImportedModule
 from nuitka.importing.PreloadedPackages import getPthImportedPackages
-from nuitka.nodes.AssignNodes import StatementAssignmentVariableName
+from nuitka.importing.StandardLibrary import isStandardLibraryPath
 from nuitka.nodes.AttributeNodes import (
     StatementAssignmentAttribute,
     makeExpressionAttributeLookup,
@@ -96,7 +95,10 @@ from nuitka.nodes.ExceptionNodes import (
 )
 from nuitka.nodes.FutureSpecs import FutureSpec
 from nuitka.nodes.GeneratorNodes import StatementGeneratorReturn
-from nuitka.nodes.ImportNodes import makeExpressionAbsoluteImportNode
+from nuitka.nodes.ImportNodes import (
+    isHardModuleWithoutSideEffect,
+    makeExpressionImportModuleFixed,
+)
 from nuitka.nodes.LoopNodes import StatementLoopBreak, StatementLoopContinue
 from nuitka.nodes.ModuleAttributeNodes import (
     ExpressionModuleAttributeFileRef,
@@ -105,8 +107,8 @@ from nuitka.nodes.ModuleAttributeNodes import (
 from nuitka.nodes.ModuleNodes import (
     CompiledPythonModule,
     CompiledPythonPackage,
+    PythonExtensionModule,
     PythonMainModule,
-    PythonShlibModule,
     makeUncompiledPythonModule,
 )
 from nuitka.nodes.NodeMakingHelpers import (
@@ -118,7 +120,10 @@ from nuitka.nodes.ReturnNodes import makeStatementReturn
 from nuitka.nodes.SliceNodes import makeExpressionBuiltinSlice
 from nuitka.nodes.StatementNodes import StatementExpressionOnly
 from nuitka.nodes.StringConcatenationNodes import ExpressionStringConcatenation
-from nuitka.nodes.VariableRefNodes import ExpressionVariableNameRef
+from nuitka.nodes.VariableNameNodes import (
+    ExpressionVariableNameRef,
+    StatementAssignmentVariableName,
+)
 from nuitka.nodes.YieldNodes import ExpressionYieldFromWaitable
 from nuitka.optimizations.BytecodeDemotion import demoteSourceCodeToBytecode
 from nuitka.Options import shallWarnUnusualCode
@@ -127,14 +132,12 @@ from nuitka.plugins.Plugins import Plugins
 from nuitka.PythonVersions import python_version
 from nuitka.Tracing import (
     general,
-    memory_logger,
     optimization_logger,
     plugins_logger,
     recursion_logger,
     unusual_logger,
 )
 from nuitka.utils import MemoryUsage
-from nuitka.utils.FileOperations import splitPath
 from nuitka.utils.ModuleNames import ModuleName
 
 from . import SyntaxErrors
@@ -193,13 +196,14 @@ from .ReformulationTryFinallyStatements import buildTryFinallyNode
 from .ReformulationWhileLoopStatements import buildWhileLoopNode
 from .ReformulationWithStatements import buildAsyncWithNode, buildWithNode
 from .ReformulationYieldExpressions import buildYieldFromNode, buildYieldNode
-from .SourceReading import (
+from .SourceHandling import (
     checkPythonVersionFromCode,
-    readSourceCodeFromFilename,
+    getSourceCodeDiff,
+    readSourceCodeFromFilenameWithInformation,
 )
 from .TreeHelpers import (
     buildNode,
-    buildNodeList,
+    buildNodeTuple,
     buildStatementsNode,
     extractDocFromBody,
     getBuildContext,
@@ -229,7 +233,6 @@ if str is not bytes:
             source_ref=source_ref,
         )
 
-
 else:
 
     def buildVariableReferenceNode(provider, node, source_ref):
@@ -418,14 +421,21 @@ def handleGlobalDeclarationNode(provider, node, source_ref):
 
         assert closure_variable.isModuleVariable()
 
+        # Special case, since Python 3.5 it is allowed to use global on the "__class__"
+        # variable as well, but it's not changing visibility of implicit "__class__" of
+        # functions, and as such it will just not be registered.
         if (
-            python_version < 0x340
-            and provider.isExpressionClassBody()
+            provider.isExpressionClassBodyBase()
             and closure_variable.getName() == "__class__"
         ):
-            SyntaxErrors.raiseSyntaxError("cannot make __class__ global", source_ref)
-
-        provider.getLocalsScope().registerClosureVariable(variable=closure_variable)
+            if python_version < 0x340:
+                SyntaxErrors.raiseSyntaxError(
+                    "cannot make __class__ global", source_ref
+                )
+        else:
+            provider.getLocalsScope().registerClosureVariable(
+                variable=closure_variable,
+            )
 
     # Drop this, not really part of our tree.
     return None
@@ -443,7 +453,7 @@ def handleNonlocalDeclarationNode(provider, node, source_ref):
     ):
         parameter_provider = parameter_provider.getParentVariableProvider()
 
-    if parameter_provider.isExpressionClassBody():
+    if parameter_provider.isExpressionClassBodyBase():
         parameter_names = ()
     else:
         parameter_names = parameter_provider.getParameters().getParameterNames()
@@ -520,7 +530,7 @@ def buildAttributeNode(provider, node, source_ref):
 
 
 def buildReturnNode(provider, node, source_ref):
-    if provider.isExpressionClassBody() or provider.isCompiledPythonModule():
+    if provider.isExpressionClassBodyBase() or provider.isCompiledPythonModule():
         SyntaxErrors.raiseSyntaxError(
             "'return' outside function", source_ref.atColumnNumber(node.col_offset)
         )
@@ -657,7 +667,7 @@ def buildFormattedValueNode(provider, node, source_ref):
 def buildJoinedStrNode(provider, node, source_ref):
     if node.values:
         return ExpressionStringConcatenation(
-            values=buildNodeList(provider, node.values, source_ref),
+            values=buildNodeTuple(provider, node.values, source_ref),
             source_ref=source_ref,
         )
     else:
@@ -774,7 +784,7 @@ def buildParseTree(provider, ast_tree, source_ref, is_module, is_main):
         if is_main and not Options.hasPythonFlagNoSite():
             statements.append(
                 StatementExpressionOnly(
-                    expression=makeExpressionAbsoluteImportNode(
+                    expression=makeExpressionImportModuleFixed(
                         module_name="site", source_ref=source_ref
                     ),
                     source_ref=source_ref,
@@ -782,9 +792,12 @@ def buildParseTree(provider, ast_tree, source_ref, is_module, is_main):
             )
 
             for path_imported_name in getPthImportedPackages():
+                if isHardModuleWithoutSideEffect(path_imported_name):
+                    continue
+
                 statements.append(
                     StatementExpressionOnly(
-                        expression=makeExpressionAbsoluteImportNode(
+                        expression=makeExpressionImportModuleFixed(
                             module_name=path_imported_name, source_ref=source_ref
                         ),
                         source_ref=source_ref,
@@ -932,23 +945,39 @@ def buildParseTree(provider, ast_tree, source_ref, is_module, is_main):
         assert False
 
 
-def decideCompilationMode(is_top, module_name, source_ref):
-    result = Plugins.decideCompilation(module_name, source_ref)
+def decideCompilationMode(is_top, module_name, for_pgo):
+    """Decide the compilation mode for a module.
+
+    module_name - The module to decide compilation mode for.
+    for_pgo - consider PGO information or not
+    """
+
+    result = Plugins.decideCompilation(module_name)
 
     # Cannot change mode of __main__ to bytecode, that is not going
     # to work currently.
     if result == "bytecode" and is_top:
         plugins_logger.warning(
             """\
-Ignoring plugin decision to compile top level package '%s'
-as bytecode, the extension module entry point is technically
+Ignoring plugin decision to compile top level package '%s' \
+as bytecode, the extension module entry point is technically \
 required to compiled."""
             % module_name
         )
         result = "compiled"
 
+    # Include all of standard library as bytecode, for now. We need to identify
+    # which ones really need that.
+    if not is_top:
+        module_filename = Importing.locateModule(
+            module_name=module_name, parent_package=None, level=0
+        )[1]
+
+        if module_filename is not None and isStandardLibraryPath(module_filename):
+            result = "bytecode"
+
     # Plugins need to win over PGO, as they might know it better
-    if result is None:
+    if result is None and not for_pgo:
         result = decideCompilationFromPGO(module_name=module_name)
 
     # Default if neither plugins nor PGO have expressed an opinion
@@ -958,120 +987,55 @@ required to compiled."""
     return result
 
 
-def _decideModuleSourceRef(filename, package, is_shlib, is_top, is_main, is_fake):
-    # Many branches due to the many cases
-    # pylint: disable=too-many-branches
-
-    assert package is None or type(package) is ModuleName
-    assert filename is not None
-
-    is_namespace = False
-    is_package = False
-
-    if is_main and os.path.isdir(filename):
-        source_filename = os.path.join(filename, "__main__.py")
-
-        if not os.path.isfile(source_filename):
-            sys.stderr.write(
-                "%s: can't find '__main__' module in '%s'\n"
-                % (os.path.basename(sys.argv[0]), filename)
-            )
-            sys.exit(2)
-
-        filename = source_filename
-
-        main_added = True
-    else:
-        main_added = False
-
-    if is_fake:
-        source_filename = filename
-
-        source_ref = SourceCodeReferences.fromFilename(filename=filename)
-
-        module_name = is_fake
-
-    elif os.path.isfile(filename):
-        source_filename = filename
-
-        source_ref = SourceCodeReferences.fromFilename(filename=filename)
-
-        if is_main:
-            module_name = ModuleName("__main__")
-        else:
-            # Derive module name from filename.
-            module_name = os.path.basename(filename)
-            if is_shlib:
-                module_name = module_name.split(".")[0]
-            elif module_name.endswith(".py"):
-                module_name = module_name[:-3]
-
-            if "." in module_name:
-                general.sysexit(
-                    "Error, '%s' is not a proper python module name.\n" % module_name
-                )
-
-            module_name = ModuleName.makeModuleNameInPackage(module_name, package)
-    elif Importing.isPackageDir(filename):
-        is_package = True
-
-        if is_top:
-            module_name = splitPath(filename)[-1]
-        else:
-            module_name = os.path.basename(filename)
-
-        module_name = ModuleName.makeModuleNameInPackage(module_name, package)
-
-        source_filename = os.path.join(filename, "__init__.py")
-
-        if not os.path.isfile(source_filename):
-            source_ref = SourceCodeReferences.fromFilename(
-                filename=filename
-            ).atInternal()
-            is_namespace = True
-        else:
-            source_ref = SourceCodeReferences.fromFilename(
-                filename=os.path.abspath(source_filename)
-            )
-
-    else:
-        sys.stderr.write(
-            "%s: can't open file '%s'.\n" % (os.path.basename(sys.argv[0]), filename)
-        )
-        sys.exit(2)
-
-    return (
-        module_name,
-        main_added,
-        is_package,
-        is_namespace,
-        source_ref,
-        source_filename,
+def _loadUncompiledModuleFromCache(module_name, is_package, source_code, source_ref):
+    result = makeUncompiledPythonModule(
+        module_name=module_name,
+        filename=source_ref.getFilename(),
+        bytecode=demoteSourceCodeToBytecode(
+            module_name=module_name,
+            source_code=source_code,
+            filename=source_ref.getFilename(),
+        ),
+        user_provided=False,
+        technical=False,
+        is_package=is_package,
     )
 
+    used_modules = OrderedSet()
+
+    used_modules = getCachedImportedModuleUsageAttempts(
+        module_name=module_name, source_code=source_code, source_ref=source_ref
+    )
+
+    # assert not is_package, (module_name, used_modules, result, result.getCompileTimeFilename())
+
+    result.setUsedModules(used_modules)
+
+    return result
+
 
 def _createModule(
     module_name,
     source_code,
     source_ref,
-    package,
-    is_shlib,
+    is_extension,
     is_namespace,
     is_package,
     is_top,
     is_main,
     main_added,
 ):
-    # Many details due to the caching done here.
-    # pylint: disable=too-many-locals
-    assert package is None or type(package) is ModuleName
-
-    if is_shlib:
-        result = PythonShlibModule(module_name=module_name, source_ref=source_ref)
+    if is_extension:
+        result = PythonExtensionModule(
+            module_name=module_name, technical=False, source_ref=source_ref
+        )
     elif is_main:
         result = PythonMainModule(
             main_added=main_added,
-            mode=decideCompilationMode(False, module_name, source_ref),
+            module_name=module_name,
+            mode=decideCompilationMode(
+                is_top=is_top, module_name=module_name, for_pgo=False
+            ),
             future_spec=None,
             source_ref=source_ref,
         )
@@ -1080,47 +1044,25 @@ def _createModule(
     elif is_namespace:
         result = createNamespacePackage(module_name, is_top, source_ref)
     else:
-        mode = decideCompilationMode(is_top, module_name, source_ref)
+        mode = decideCompilationMode(
+            is_top=is_top, module_name=module_name, for_pgo=False
+        )
 
         if (
             mode == "bytecode"
             and not is_top
-            and hasCachedImportedModulesNames(module_name, source_code)
+            and not Options.shallDisableBytecodeCacheUsage()
+            and hasCachedImportedModuleUsageAttempts(
+                module_name=module_name, source_code=source_code, source_ref=source_ref
+            )
         ):
-
-            optimization_logger.info(
-                "'%s' is included as bytecode." % (module_name.asString())
-            )
-            result = makeUncompiledPythonModule(
+            result = _loadUncompiledModuleFromCache(
                 module_name=module_name,
-                filename=source_ref.getFilename(),
-                bytecode=demoteSourceCodeToBytecode(
-                    module_name=module_name,
-                    source_code=source_code,
-                    filename=source_ref.getFilename(),
-                ),
-                user_provided=False,
-                technical=False,
                 is_package=is_package,
+                source_code=source_code,
+                source_ref=source_ref,
             )
 
-            used_modules = OrderedSet()
-
-            for used_module_name in getCachedImportedModulesNames(
-                module_name=module_name, source_code=source_code
-            ):
-                (_module_package, module_filename, _finding,) = Importing.findModule(
-                    importing=result,
-                    module_name=used_module_name,
-                    parent_package=None,
-                    level=-1,
-                    warn=False,
-                )
-
-                used_modules.add((used_module_name, os.path.relpath(module_filename)))
-
-            result.setUsedModules(used_modules)
-
             # Not used anymore
             source_code = None
         else:
@@ -1159,35 +1101,41 @@ def createModuleTree(module, source_ref, ast_tree, is_main):
     if module_body.isStatementsFrame():
         module_body = makeStatementsSequenceFromStatement(statement=module_body)
 
-    module.setChild("body", module_body)
+    module.setChildBody(module_body)
 
     completeVariableClosures(module)
 
     if Options.isShowMemory():
-        memory_watch.finish()
-
-        memory_logger.info(
-            "Memory usage changed loading module '%s': %s"
-            % (module.getFullName(), memory_watch.asStr())
+        memory_watch.finish(
+            "Memory usage changed loading module '%s'" % module.getFullName()
         )
 
 
-def buildMainModuleTree(filename, package, is_main):
+def buildMainModuleTree(filename, is_main, source_code):
     # Detect to be frozen modules if any, so we can consider to not follow
     # to them.
 
+    if is_main:
+        # TODO: Doesn't work for deeply nested packages at all.
+        if Options.hasPythonFlagPackageMode():
+            module_name = ModuleName(os.path.basename(filename) + ".__main__")
+        else:
+            module_name = ModuleName("__main__")
+    else:
+        module_name = Importing.getModuleNameAndKindFromFilename(filename)[0]
+
     module, _added = buildModule(
+        module_name=module_name,
         module_filename=filename,
-        module_package=package,
-        source_code=None,
+        source_code=source_code,
         is_top=True,
         is_main=is_main,
-        is_shlib=False,
-        is_fake=False,
+        is_extension=False,
+        is_fake=source_code is not None,
         hide_syntax_error=False,
     )
 
-    if Options.isStandaloneMode() and is_main:
+    if is_main and Options.isStandaloneMode():
         module.setEarlyModules(detectEarlyImports())
 
     # Main modules do not get added to the import cache, but plugins get to see it.
@@ -1200,14 +1148,12 @@ def buildMainModuleTree(filename, package, is_main):
 
 
 def _makeModuleBodyFromSyntaxError(exc, module_name, module_filename):
-    assert module_name != "markupsafe._speedups", module_filename
-
     if module_filename not in Importing.warned_about:
         Importing.warned_about.add(module_filename)
 
         recursion_logger.warning(
             """\
-Cannot follow import to module '%s' because of %r."""
+Cannot follow import to module '%s' because of '%s'."""
             % (module_name, exc.__class__.__name__)
         )
 
@@ -1232,7 +1178,7 @@ Cannot follow import to module '%s' because of %r."""
     )
 
     module_body = makeStatementsSequenceFromStatement(statement=module_body)
-    module.setChild("body", module_body)
+    module.setChildBody(module_body)
 
     return module
 
@@ -1243,7 +1189,7 @@ def _makeModuleBodyTooComplex(module_name, module_filename, source_code, is_pack
 
         recursion_logger.warning(
             """\
-Cannot follow import to import module '%r' ('%r') because code is too complex."""
+Cannot follow import to import module '%r' ('%s') because code is too complex."""
             % (
                 module_name,
                 module_filename,
@@ -1265,55 +1211,70 @@ Cannot follow import to import module '%r' ('%r') because code is too complex.""
 
 
 def buildModule(
+    module_name,
     module_filename,
-    module_package,
     source_code,
     is_top,
     is_main,
-    is_shlib,
+    is_extension,
     is_fake,
     hide_syntax_error,
 ):
-    # Many details to deal with, pylint: disable=too-many-locals
-
+    # Many details to deal with,
+    # pylint: disable=too-many-branches,too-many-locals,too-many-statements
     (
-        module_name,
         main_added,
         is_package,
         is_namespace,
         source_ref,
         source_filename,
-    ) = _decideModuleSourceRef(
+    ) = Importing.decideModuleSourceRef(
         filename=module_filename,
-        package=module_package,
-        is_top=is_top,
+        module_name=module_name,
         is_main=is_main,
-        is_shlib=is_shlib,
         is_fake=is_fake,
+        logger=general,
     )
 
+    if Options.hasPythonFlagPackageMode():
+        if is_top and Options.shallMakeModule():
+            optimization_logger.warning(
+                "Python flag -m (package_mode) has no effect in module mode, it's only for executables."
+            )
+        elif is_main and not main_added:
+            optimization_logger.warning(
+                "Python flag -m (package_mode) only works on packages with '__main__.py'."
+            )
+
     # Read source code if necessary. Might give a SyntaxError due to not being proper
     # encoded source.
-    if source_filename is not None and not is_namespace and not is_shlib:
-        try:
-            # For fake modules, source is provided directly.
-            if source_code is None:
-                source_code = readSourceCodeFromFilename(
+    if source_filename is not None and not is_namespace and not is_extension:
+        # For fake modules, source is provided directly.
+        original_source_code = None
+        contributing_plugins = ()
+
+        if source_code is None:
+            try:
+                (
+                    source_code,
+                    original_source_code,
+                    contributing_plugins,
+                ) = readSourceCodeFromFilenameWithInformation(
                     module_name=module_name, source_filename=source_filename
                 )
-        except SyntaxError as e:
-            # Avoid hiding our own syntax errors.
-            if not hasattr(e, "generated_by_nuitka"):
-                raise
+            except SyntaxError as e:
+                # Avoid hiding our own syntax errors.
+                if not hasattr(e, "generated_by_nuitka"):
+                    raise
 
-            # Do not hide SyntaxError in main module.
-            if not hide_syntax_error:
-                raise
+                # Do not hide SyntaxError in main module.
+                if not hide_syntax_error:
+                    raise
 
-            module = _makeModuleBodyFromSyntaxError(
-                exc=e, module_name=module_name, module_filename=module_filename
-            )
-            return module, True
+                module = _makeModuleBodyFromSyntaxError(
+                    exc=e, module_name=module_name, module_filename=module_filename
+                )
+                return module, True
 
         try:
             ast_tree = parseSourceCodeToAst(
@@ -1327,6 +1288,34 @@ def buildModule(
             if not hide_syntax_error:
                 raise
 
+            if original_source_code is not None:
+                try:
+                    parseSourceCodeToAst(
+                        source_code=original_source_code,
+                        module_name=module_name,
+                        filename=source_filename,
+                        line_offset=0,
+                    )
+                except (SyntaxError, IndentationError):
+                    # Also an exception without the plugins, that is OK
+                    pass
+                else:
+                    source_diff = getSourceCodeDiff(original_source_code, source_code)
+
+                    for line in source_diff:
+                        plugins_logger.warning(line)
+
+                    if len(contributing_plugins) == 1:
+                        contributing_plugins[0].sysexit(
+                            "Making changes to '%s' that cause SyntaxError '%s'"
+                            % (module_name, e)
+                        )
+                    else:
+                        plugins_logger.sysexit(
+                            "One of the plugins '%s' is making changes to '%s' that cause SyntaxError '%s'"
+                            % (",".join(contributing_plugins), module_name, e)
+                        )
+
             module = _makeModuleBodyFromSyntaxError(
                 exc=e, module_name=module_name, module_filename=module_filename
             )
@@ -1349,12 +1338,11 @@ def buildModule(
 
     module = _createModule(
         module_name=module_name,
-        package=module_package,
         source_code=source_code,
         source_ref=source_ref,
         is_top=is_top,
         is_main=is_main,
-        is_shlib=is_shlib,
+        is_extension=is_extension,
         is_namespace=is_namespace,
         is_package=is_package,
         main_added=main_added,
