@@ -1,4 +1,4 @@
-#     Copyright 2021, Kay Hayen, mailto:kay.hayen@gmail.com
+#     Copyright 2022, Kay Hayen, mailto:kay.hayen@gmail.com
 #
 #     Part of "Nuitka", an optimizing Python compiler that is compatible and
 #     integrates with CPython, but also works on its own.
@@ -19,10 +19,6 @@
 
 One for each type of call. """
 
-from nuitka.nodes.AssignNodes import (
-    StatementAssignmentVariable,
-    StatementReleaseVariable,
-)
 from nuitka.nodes.AttributeNodes import (
     ExpressionAttributeCheck,
     makeExpressionAttributeLookup,
@@ -57,9 +53,9 @@ from nuitka.nodes.ExceptionNodes import (
 )
 from nuitka.nodes.FunctionAttributeNodes import ExpressionFunctionErrorStr
 from nuitka.nodes.FunctionNodes import (
-    ExpressionFunctionCall,
-    ExpressionFunctionCreation,
     ExpressionFunctionRef,
+    makeExpressionFunctionCall,
+    makeExpressionFunctionCreation,
 )
 from nuitka.nodes.LoopNodes import StatementLoop, StatementLoopBreak
 from nuitka.nodes.OperatorNodes import makeBinaryOperationNode
@@ -72,10 +68,12 @@ from nuitka.nodes.TypeNodes import (
     ExpressionBuiltinIsinstance,
     ExpressionBuiltinType1,
 )
+from nuitka.nodes.VariableAssignNodes import makeStatementAssignmentVariable
 from nuitka.nodes.VariableRefNodes import (
     ExpressionTempVariableRef,
     ExpressionVariableRef,
 )
+from nuitka.nodes.VariableReleaseNodes import makeStatementReleaseVariable
 from nuitka.PythonVersions import (
     getComplexCallSequenceErrorTemplate,
     python_version,
@@ -192,7 +190,7 @@ def getCallableNameDescBody():
     )
 
     if python_version >= 0x390:
-        result.setChild("body", functions_case)
+        result.setChildBody(functions_case)
 
         return result
 
@@ -290,8 +288,7 @@ def getCallableNameDescBody():
     else:
         normal_cases = ("function", "builtin_function_or_method")
 
-    result.setChild(
-        "body",
+    result.setChildBody(
         makeStatementsSequenceFromStatement(
             statement=makeStatementConditional(
                 condition=ExpressionBuiltinIsinstance(
@@ -314,7 +311,7 @@ def getCallableNameDescBody():
                 no_branch=no_branch,
                 source_ref=internal_source_ref,
             )
-        ),
+        )
     )
 
     return result
@@ -334,8 +331,8 @@ def makeStarListArgumentErrorRaise(called_variable, star_list_variable):
                     ),
                     right=makeExpressionMakeTuple(
                         elements=(
-                            ExpressionFunctionCall(
-                                function=ExpressionFunctionCreation(
+                            makeExpressionFunctionCall(
+                                function=makeExpressionFunctionCreation(
                                     function_ref=ExpressionFunctionRef(
                                         function_body=getCallableNameDescBody(),
                                         source_ref=internal_source_ref,
@@ -397,7 +394,7 @@ def _makeStarListArgumentToTupleStatement(called_variable, star_list_variable):
                 ),
                 source_ref=internal_source_ref,
             ),
-            yes_branch=StatementAssignmentVariable(
+            yes_branch=makeStatementAssignmentVariable(
                 variable=star_list_variable,
                 source=ExpressionBuiltinTuple(
                     value=ExpressionVariableRef(
@@ -414,7 +411,7 @@ def _makeStarListArgumentToTupleStatement(called_variable, star_list_variable):
         )
     else:
         non_tuple_code = makeTryExceptSingleHandlerNode(
-            tried=StatementAssignmentVariable(
+            tried=makeStatementAssignmentVariable(
                 variable=star_list_variable,
                 source=ExpressionBuiltinTuple(
                     value=ExpressionVariableRef(
@@ -465,8 +462,8 @@ def _makeRaiseExceptionMustBeMapping(called_variable, star_dict_variable):
                     ),
                     right=makeExpressionMakeTuple(
                         elements=(
-                            ExpressionFunctionCall(
-                                function=ExpressionFunctionCreation(
+                            makeExpressionFunctionCall(
+                                function=makeExpressionFunctionCreation(
                                     function_ref=ExpressionFunctionRef(
                                         function_body=getCallableNameDescBody(),
                                         source_ref=internal_source_ref,
@@ -511,7 +508,7 @@ def _makeRaiseExceptionMustBeMapping(called_variable, star_dict_variable):
 def _makeIteratingLoopStatement(tmp_iter_variable, tmp_item_variable, statements):
     loop_body = makeStatementsSequenceFromStatements(
         makeTryExceptSingleHandlerNode(
-            tried=StatementAssignmentVariable(
+            tried=makeStatementAssignmentVariable(
                 variable=tmp_item_variable,
                 source=ExpressionBuiltinNext1(
                     value=ExpressionTempVariableRef(
@@ -584,7 +581,7 @@ def _makeStarDictArgumentToDictStatement(result, called_variable, star_dict_vari
 
     mapping_case = makeStatementsSequenceFromStatements(
         makeTryExceptSingleHandlerNode(
-            tried=StatementAssignmentVariable(
+            tried=makeStatementAssignmentVariable(
                 variable=tmp_keys_variable,
                 source=makeCallNode(
                     _makeNameAttributeLookup(
@@ -603,7 +600,7 @@ def _makeStarDictArgumentToDictStatement(result, called_variable, star_dict_vari
             ),
             source_ref=internal_source_ref,
         ),
-        StatementAssignmentVariable(
+        makeStatementAssignmentVariable(
             variable=tmp_iter_variable,
             source=ExpressionBuiltinIter1(
                 value=ExpressionTempVariableRef(
@@ -613,7 +610,7 @@ def _makeStarDictArgumentToDictStatement(result, called_variable, star_dict_vari
             ),
             source_ref=internal_source_ref,
         ),
-        StatementAssignmentVariable(
+        makeStatementAssignmentVariable(
             variable=tmp_dict_variable,
             source=makeConstantRefNode(
                 constant={}, source_ref=internal_source_ref, user_provided=True
@@ -625,7 +622,7 @@ def _makeStarDictArgumentToDictStatement(result, called_variable, star_dict_vari
             tmp_item_variable=tmp_key_variable,
             statements=loop_body,
         ),
-        StatementAssignmentVariable(
+        makeStatementAssignmentVariable(
             variable=star_dict_variable,
             source=ExpressionTempVariableRef(
                 variable=tmp_dict_variable, source_ref=internal_source_ref
@@ -653,16 +650,16 @@ def _makeStarDictArgumentToDictStatement(result, called_variable, star_dict_vari
     )
 
     final = (
-        StatementReleaseVariable(
+        makeStatementReleaseVariable(
             variable=tmp_dict_variable, source_ref=internal_source_ref
         ),
-        StatementReleaseVariable(
+        makeStatementReleaseVariable(
             variable=tmp_iter_variable, source_ref=internal_source_ref
         ),
-        StatementReleaseVariable(
+        makeStatementReleaseVariable(
             variable=tmp_keys_variable, source_ref=internal_source_ref
         ),
-        StatementReleaseVariable(
+        makeStatementReleaseVariable(
             variable=tmp_key_variable, source_ref=internal_source_ref
         ),
     )
@@ -681,8 +678,8 @@ def _makeRaiseNoStringItem(called_variable):
                 source_ref=internal_source_ref,
                 user_provided=True,
             ),
-            right=ExpressionFunctionCall(
-                function=ExpressionFunctionCreation(
+            right=makeExpressionFunctionCall(
+                function=makeExpressionFunctionCreation(
                     function_ref=ExpressionFunctionRef(
                         function_body=getCallableNameDescBody(),
                         source_ref=internal_source_ref,
@@ -736,8 +733,8 @@ def _makeRaiseDuplicationItem(called_variable, tmp_key_variable):
                     ),
                     right=makeExpressionMakeTuple(
                         elements=(
-                            ExpressionFunctionCall(
-                                function=ExpressionFunctionCreation(
+                            makeExpressionFunctionCall(
+                                function=makeExpressionFunctionCreation(
                                     function_ref=ExpressionFunctionRef(
                                         function_body=getCallableNameDescBody(),
                                         source_ref=internal_source_ref,
@@ -785,13 +782,13 @@ def _makeStarDictArgumentMergeToKwStatement(
     tmp_key_variable = result.allocateTempVariable(temp_scope, "key_xxx")
 
     final = [
-        StatementReleaseVariable(
+        makeStatementReleaseVariable(
             variable=tmp_iter_variable, source_ref=internal_source_ref
         ),
-        StatementReleaseVariable(
+        makeStatementReleaseVariable(
             variable=tmp_keys_variable, source_ref=internal_source_ref
         ),
-        StatementReleaseVariable(
+        makeStatementReleaseVariable(
             variable=tmp_key_variable, source_ref=internal_source_ref
         ),
     ]
@@ -835,7 +832,7 @@ def _makeStarDictArgumentMergeToKwStatement(
 
     mapping_case = makeStatementsSequenceFromStatements(
         makeTryExceptSingleHandlerNode(
-            tried=StatementAssignmentVariable(
+            tried=makeStatementAssignmentVariable(
                 variable=tmp_keys_variable,
                 source=makeCallNode(
                     _makeNameAttributeLookup(
@@ -854,7 +851,7 @@ def _makeStarDictArgumentMergeToKwStatement(
             ),
             source_ref=internal_source_ref,
         ),
-        StatementAssignmentVariable(
+        makeStatementAssignmentVariable(
             variable=tmp_iter_variable,
             source=ExpressionBuiltinIter1(
                 value=ExpressionTempVariableRef(
@@ -878,19 +875,19 @@ def _makeStarDictArgumentMergeToKwStatement(
     tmp_key_variable = result.allocateTempVariable(temp_scope, "key")
 
     final += (
-        StatementReleaseVariable(
+        makeStatementReleaseVariable(
             variable=tmp_iter_variable, source_ref=internal_source_ref
         ),
-        StatementReleaseVariable(
+        makeStatementReleaseVariable(
             variable=tmp_item_variable, source_ref=internal_source_ref
         ),
-        StatementReleaseVariable(
+        makeStatementReleaseVariable(
             variable=tmp_key_variable, source_ref=internal_source_ref
         ),
     )
 
     dict_loop_body = (
-        StatementAssignmentVariable(
+        makeStatementAssignmentVariable(
             variable=tmp_key_variable,
             source=ExpressionSubscriptLookup(
                 expression=ExpressionTempVariableRef(
@@ -940,7 +937,7 @@ def _makeStarDictArgumentMergeToKwStatement(
     )
 
     dict_case = makeStatementsSequenceFromStatements(
-        StatementAssignmentVariable(
+        makeStatementAssignmentVariable(
             variable=kw_variable,
             source=ExpressionBuiltinDict(
                 pos_arg=ExpressionVariableRef(
@@ -951,7 +948,7 @@ def _makeStarDictArgumentMergeToKwStatement(
             ),
             source_ref=internal_source_ref,
         ),
-        StatementAssignmentVariable(
+        makeStatementAssignmentVariable(
             variable=tmp_iter_variable,
             source=ExpressionBuiltinIter1(
                 value=ExpressionDictOperationIteritems(
@@ -1061,7 +1058,7 @@ def getFunctionCallHelperStarList():
         ),
     )
 
-    result.setChild("body", body)
+    result.setChildBody(body)
 
     return result
 
@@ -1130,7 +1127,7 @@ def getFunctionCallHelperKeywordsStarList():
         ),
     )
 
-    result.setChild("body", body)
+    result.setChildBody(body)
 
     return result
 
@@ -1204,7 +1201,7 @@ def getFunctionCallHelperPosStarList():
         ),
     )
 
-    result.setChild("body", body)
+    result.setChildBody(body)
 
     return result
 
@@ -1281,7 +1278,7 @@ def getFunctionCallHelperPosKeywordsStarList():
         ),
     )
 
-    result.setChild("body", body)
+    result.setChildBody(body)
 
     return result
 
@@ -1360,7 +1357,7 @@ def getFunctionCallHelperStarDict():
         ),
     )
 
-    result.setChild("body", body)
+    result.setChildBody(body)
 
     return result
 
@@ -1444,7 +1441,7 @@ def getFunctionCallHelperPosStarDict():
         ),
     )
 
-    result.setChild("body", body)
+    result.setChildBody(body)
 
     return result
 
@@ -1552,7 +1549,7 @@ def getFunctionCallHelperKeywordsStarDict():
         ),
     )
 
-    result.setChild("body", body)
+    result.setChildBody(body)
 
     return result
 
@@ -1664,7 +1661,7 @@ def getFunctionCallHelperPosKeywordsStarDict():
         ),
     )
 
-    result.setChild("body", body)
+    result.setChildBody(body)
 
     return result
 
@@ -1759,7 +1756,7 @@ def getFunctionCallHelperStarListStarDict():
 
     body = makeStatementsSequenceFromStatements(*statements)
 
-    result.setChild("body", body)
+    result.setChildBody(body)
 
     return result
 
@@ -1833,7 +1830,7 @@ def getFunctionCallHelperPosStarListStarDict():
 
     body = makeStatementsSequenceFromStatements(*statements)
 
-    result.setChild("body", body)
+    result.setChildBody(body)
 
     return result
 
@@ -1897,7 +1894,7 @@ def getFunctionCallHelperKeywordsStarListStarDict():
 
     body = makeStatementsSequenceFromStatements(*statements)
 
-    result.setChild("body", body)
+    result.setChildBody(body)
 
     return result
 
@@ -1975,7 +1972,7 @@ def getFunctionCallHelperPosKeywordsStarListStarDict():
 
     body = makeStatementsSequenceFromStatements(*statements)
 
-    result.setChild("body", body)
+    result.setChildBody(body)
 
     return result
 
@@ -2065,7 +2062,7 @@ def getFunctionCallHelperDictionaryUnpacking():
     loop_body = (
         makeTryExceptSingleHandlerNode(
             tried=makeStatementsSequenceFromStatements(
-                StatementAssignmentVariable(
+                makeStatementAssignmentVariable(
                     variable=tmp_iter2_variable,
                     source=ExpressionBuiltinIter1(
                         value=makeCallNode(
@@ -2097,25 +2094,25 @@ def getFunctionCallHelperDictionaryUnpacking():
     )
 
     final = (
-        StatementReleaseVariable(
+        makeStatementReleaseVariable(
             variable=tmp_result_variable, source_ref=internal_source_ref
         ),
-        StatementReleaseVariable(
+        makeStatementReleaseVariable(
             variable=tmp_iter_variable, source_ref=internal_source_ref
         ),
-        StatementReleaseVariable(
+        makeStatementReleaseVariable(
             variable=tmp_item_variable, source_ref=internal_source_ref
         ),
-        StatementReleaseVariable(
+        makeStatementReleaseVariable(
             variable=tmp_iter2_variable, source_ref=internal_source_ref
         ),
-        StatementReleaseVariable(
+        makeStatementReleaseVariable(
             variable=tmp_key_variable, source_ref=internal_source_ref
         ),
     )
 
     tried = makeStatementsSequenceFromStatements(
-        StatementAssignmentVariable(
+        makeStatementAssignmentVariable(
             variable=tmp_iter_variable,
             source=ExpressionBuiltinIter1(
                 value=ExpressionVariableRef(
@@ -2125,7 +2122,7 @@ def getFunctionCallHelperDictionaryUnpacking():
             ),
             source_ref=internal_source_ref,
         ),
-        StatementAssignmentVariable(
+        makeStatementAssignmentVariable(
             variable=tmp_result_variable,
             source=makeConstantRefNode(constant={}, source_ref=internal_source_ref),
             source_ref=internal_source_ref,
@@ -2152,6 +2149,6 @@ def getFunctionCallHelperDictionaryUnpacking():
         )
     )
 
-    result.setChild("body", body)
+    result.setChildBody(body)
 
     return result
