@@ -45,6 +45,14 @@ def makeTriggerModuleName(module_name, trigger_name):
     return ModuleName(module_name + trigger_name)
 
 
+# Multidist prefix
+_multi_dist_prefix = "multidist-"
+
+
+def makeMultidistModuleName(count, suffix):
+    return ModuleName("%s%d-%s" % (_multi_dist_prefix, count, suffix))
+
+
 class ModuleName(str):
     def __init__(self, value):
         assert checkModuleName(value), value
@@ -219,13 +227,13 @@ class ModuleName(str):
         """
 
         if self == pattern:
-            return True, "is exact match of %r" % pattern
+            return True, "is exact match of '%s'" % pattern
         elif self.isBelowNamespace(pattern):
-            return True, "is package content of %r" % pattern
+            return True, "is package content of '%s'" % pattern
         elif fnmatch.fnmatch(self.asString(), pattern):
-            return True, "matches pattern %r" % pattern
+            return True, "matches pattern '%s'" % pattern
         elif fnmatch.fnmatch(self.asString(), pattern + ".*"):
-            return True, "is package content of match to pattern %r" % pattern
+            return True, "is package content of match to pattern '%s'" % pattern
         else:
             return False, None
 
@@ -255,9 +263,12 @@ class ModuleName(str):
     def isFakeModuleName(self):
         return str(self).endswith(trigger_names)
 
+    def isMultidistModuleName(self):
+        return str(self).startswith(_multi_dist_prefix)
+
     # Reject APIs being used. TODO: Maybe make this a decorator for reuse.
     # TODO: Add rsplit and subscript operations too.
-    for _func_name in ("split", "startswith", "endswith"):
+    for _func_name in ("split", "startswith", "endswith", "__mod__"):
         code = """\
 def %(func_name)s(*args, **kwargs):
     from nuitka.Errors import NuitkaCodeDeficit
